#!/bin/sh
# This script must be able to run on busybox-powered devices, so uses POSIX shell instead of bash.

set -e #< Exit on error.

CUSTOMIZATION="networkoptix"
DISTRIB="nxgo-server-6.0.0.39148-linux_arm64.hanwha_edge1-rc"

INSTALL_DIR="/opt/${CUSTOMIZATION}"
MODULE_DIR="${INSTALL_DIR}/mediaserver"
SERVER_PLUGINS_DIR="${MODULE_DIR}/bin/plugins"
BUNDLED_PLUGINS_FILE="${MODULE_DIR}/bundled_plugins.txt"
TAR_FILE="./${DISTRIB}.tar.gz"
BACKUP_PLUGINS_DIR=""

CONTROL_SCRIPT="${MODULE_DIR}/bin/mediaserver.sh"
CONFIG_HELPER_SCRIPT="${MODULE_DIR}/bin/config_helper.sh"

version_cmp()
{
    version1="$1"
    version2="$2"

    for i in $(seq 1 4); do
        version1_component="$(echo "${version1}" | cut -d "." -f $i)"
        version2_component="$(echo "${version2}" | cut -d "." -f $i)"
        if [ "${version1_component}" -gt "${version2_component}" ]; then
            echo "1"
            return
        elif [ "${version1_component}" -lt "${version2_component}" ]; then
            echo "-1"
            return
        fi
    done

    echo "0"
}

ensure_update_to_higher_version()
{
    # Absence of this script means either no Server is installed or the Server installation is
    # broken. In both cases we should continue with the installation process.
    if [ ! -f "${CONFIG_HELPER_SCRIPT}" ]; then
        return
    fi

    # The path to build_info.txt in the tar file is the same that in the file system except for
    # the leading "/", so sed is used to remove the first char from the file system path to get
    # the archive path.
    build_info_path="$(echo "${MODULE_DIR}/build_info.txt" | sed 's:^/::' -)"

    # Here --transform extracts build_info.txt without its path.
    tar xvf "${TAR_FILE}" -C /tmp "${build_info_path}" --transform='s/.*\///' >/dev/null 2>&1

    old_version="$("${CONFIG_HELPER_SCRIPT}" "${MODULE_DIR}/build_info.txt" version)"
    new_version="$("${CONFIG_HELPER_SCRIPT}" /tmp/build_info.txt version)"

    if [ "$(version_cmp "${old_version}" "${new_version}")" -gt 0 ]; then
        echo "Newer version is already installed: ${old_version} >= ${new_version}" >&2
        exit 1
    fi
}

remove_bundled_plugins()
{
    if [ ! -d "${SERVER_PLUGINS_DIR}" ]; then
        return
    fi

    if [ ! -f "${BUNDLED_PLUGINS_FILE}" ]; then
        # Only the old versions of the Server (up to 5.1.4) do not generate the bundled_plugins.txt
        # file, so we create it for them right now. Remove this when the support for these versions
        # is dropped.
        cat <<EOF > "${BUNDLED_PLUGINS_FILE}"
libgeneric_multicast_plugin.so
libgeneric_rtsp_plugin.so
libhanwha_analytics_plugin.so
libmjpeg_link_plugin.so
libusb_camera_plugin.so
EOF
    fi

    xargs -I{} rm -rf "${SERVER_PLUGINS_DIR}/{}" < "${BUNDLED_PLUGINS_FILE}"
}

back_up_necessary_files()
{
    # Backup 3rd-party plugins.
    if [ -d "${SERVER_PLUGINS_DIR}" ]; then
        BACKUP_PLUGINS_DIR="$(mktemp -d)"
        cp -r "${SERVER_PLUGINS_DIR}/"* "${BACKUP_PLUGINS_DIR}"
    fi
}

restore_backup_files()
{
    if [ -d "${BACKUP_PLUGINS_DIR}" ]; then
        # Restore plugin only if there is no such plugin in the target directory (i.e. no such
        # plugin is bundled with the new version), thus "-n" flag for "mv" command.
        mv -n "${BACKUP_PLUGINS_DIR}"/* "${SERVER_PLUGINS_DIR}"
        rm -Rf "${BACKUP_PLUGINS_DIR}"
    fi
}

create_bundled_plugins_list()
{
    echo "# List of plugins supplied with the Server package" > "${BUNDLED_PLUGINS_FILE}"
    ( #< Use subscript to avoid changing the current directory.
        cd "${SERVER_PLUGINS_DIR}"
        for plugin in *; do
            echo "${plugin}" >> "${BUNDLED_PLUGINS_FILE}"
        done
    )
}

upgrade_vms()
{
    if [ -f "${CONTROL_SCRIPT}" ]; then
        if "${CONTROL_SCRIPT}" status >/dev/null 2>&1; then
            was_running="1"
            "${CONTROL_SCRIPT}" stop
        else
            was_running="0"
        fi
    else
        was_running="0"
    fi

    remove_bundled_plugins
    back_up_necessary_files

    rm -rf "${MODULE_DIR}/lib"
    rm -rf "${MODULE_DIR}/bin/plugins"
    rm -rf "${MODULE_DIR}/bin/plugins_optional"
    mkdir -p "${INSTALL_DIR}"

    tar xfv "${TAR_FILE}" --dereference --directory "/"

    create_bundled_plugins_list

    mkdir -p "${INSTALL_DIR}/mediaserver/etc"

    restore_backup_files

    "${INSTALL_DIR}/mediaserver/bin/upgrade_config.sh" \
        "${INSTALL_DIR}/mediaserver/etc/mediaserver.conf" \
        "${INSTALL_DIR}/mediaserver/share/configs/mediaserver.conf" \
        "${INSTALL_DIR}/mediaserver/etc/mediaserver_conf_added_keys.txt" \
        "${CONFIG_HELPER_SCRIPT}"

    if [ "${was_running}" = "1" ]; then
        "${CONTROL_SCRIPT}" start
    fi
}

main()
{
    ensure_update_to_higher_version

    if [ ! -z "$1" ]; then
        upgrade_vms >>"$1" 2>&1
    else
        upgrade_vms 2>&1
    fi
}

main "$@"
